/*
 *      $Filename: support.c $
 *      $Revision: 3.8 $
 *      $Date: 1996/06/28 19:58:43 $
 *
 *      Copyright (C) 1994-96 by Peter Simons <simons@peti.rhein.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: support.c,v 3.8 1996/06/28 19:58:43 simons Exp $
 *
 */

/************************************* includes ***********/
#include "includes.h"

/************************************* defines ************/

/************************************* prototypes *********/
#include "support.h"

/************************************* global variables ***/
static char rcssid[] = "$Id: support.c,v 3.8 1996/06/28 19:58:43 simons Exp $";
int       debuglevel = 0;

/************************************* routines ***********/

/****i***************************************************************************
NAME
     log, setdebuglevel, debug - easy system logging

SYNOPSIS
     #include <support.h>

     void
     log(int priority, const char *message, ...)

     void
     logerrno(int priority, const char *fmt, ...)

     void
     debug(int level, const char *message, ...)

     void
     setdebuglevel(int debuglevel)

DESCRIPTION
     The three routines are kind of front ends for the syslog(3)
     family of functions, common under Unix and many other operating
     systems. In fact one could simply use syslog() directly to
     archive the same effect, but unfortunately many implementations
     differ in the amount of functionality they provide. This is why
     to use these front-ends as it is well defined what they do and
     what not. If your system supports syslog(), the routine will be
     used in the end, but if it doesn't your program will still work.

     log() is the main routine you should call to log all kind of
     errors, warnings or notes that are important for the user of your
     program. It requires at least two parameters: The priority of the
     logged message and the message itself. Priorities will be
     described later. The message may be a format string as accepted
     by printf(3) and many similar routines, followed by an arbitrary
     number of parameters, which will be inserted in the string itself
     according to the placeholders '%s', '%d' and others. Please refer
     to printf(3) for a full description of this mechanism.

     logerrno() is a special case of the log() routine, which is
     designed to log the error messages returned through errno(2). The
     usage is very similar to log(), but you might want to have a look
     at the example section below. To describe it quickly, logerrno()
     will look up the description of the error using the errno
     variable. Then it will append a colon (':') to your string and
     append the error description behind that.

     debug() is a special kind of the log(LOG_DEBUG, ...) call.
     Effectively, it does absolutely the same as calling log() with
     the debug priority, but it is sometimes preferable to habe debug
     messages cleary seperated from other log messages. Furthermore,
     using debug() instead of log() has the advantage that the
     debugging output is not included in your binary, when the NDEBUG
     flag is defined at compile time.

     debug() expects the level of the debugging message as the first
     parameter. The meaning is as follows: Debugging output may have
     different priotities, too. Some messages are quite interesting to
     see what the program does while others are very low-level and
     fill quickly fill your harddisk up when logged all the time.
     The priority of the debug messages ranges from 0 to 3, 0 being
     the most important and 3 the least. You should not really use
     level 0, though, as these messages will be display always and
     cannot be masked through setdebuglevel(). If you're message is
     that important, rather log() it with a minor priority.

     setdebuglevel() controls the amount of debugging output given
     from debug(). The value you provide will be checked by all
     following debug() calls. If the level given to debug() is higher
     than the debuglevel you set, then the message will be surpressed.
     Hence, setting the debuglevel to zero (0) will surpress all
     debugging output. The debuglevel can be changed several times
     during the program execution, so you can debug only certain parts
     of your program.

     Please note that you should call openlog(3) before using any of
     these routines, because this call will specify the facility, the
     log messages should go to.

PRIORITIES
     These are the priorities understood by the log() call, ranking
     from the highest to the lowest:

     LOG_EMERG     A panic condition.  This is normally broadcast to all
                   users.

     LOG_ALERT     A condition that should be corrected immediately, such
                   as a corrupted system database.

     LOG_CRIT      Critical conditions, e.g., hard device errors.

     LOG_ERR       Errors.

     LOG_WARNING   Warning messages.

     LOG_NOTICE    Conditions that are not error conditions, but
                   should possibly be handled specially.

     LOG_INFO      Informational messages.

     LOG_DEBUG     Messages that contain information normally of use
                   only when debugging a program.

EXAMPLES
          FILE *fh;

          if (fh = fopen("/etc/passwd", "r")) {
	       [do whatever appropriate]
          }
	  else
	       logerrno(LOG_ERR, "fopen(%s) failed", "/etc/passwd");

     In case the fopen() call fails, the result in the logfile will
     probably be something like this:

      | some time, some machine: fopen(/etc/passwd) failed: permission denied

BUGS
     Currently, my routines do not work without syslog() at all, what
     is somewhat a drawback as their only purpose was to make the
     programmer independent of syslog().

     It is not a good thing(tm) that the programmer has to call
     openlog(), which may very well not be available on some systems.

SEE ALSO
     syslog(3), openlog(3), errno(2)
********************************************************************************/

void 
setdebuglevel(int level)
{
    assert(level >= 0 && level <= 3);
    debuglevel = level;
    switch (level) {
    case 3:
	debug(1, "Setting debug output to 'all'.");
	break;
    case 2:
	debug(1, "Setting debug output to 'much'.");
	break;
    case 1:
	debug(1, "Setting debug output to 'important'.");
	break;
    case 0:
	break;
    }
}

#ifdef HAVE_VSYSLOG
void 
log(int priority, const char *fmt,...)
{
    va_list   ap;

    va_start(ap, fmt);
    vsyslog(priority, fmt, ap);
    va_end(ap);
}

#else
#ifdef HAVE_SYSLOG
#define log syslog
#else
void 
log(int priority, const char *fmt,...)
{
}

#endif
#endif

void 
debug(int level, const char *fmt,...)
{
#ifdef HAVE_VSYSLOG
    assert(level >= 0 && level <= 3);
    if (level <= debuglevel) {
	va_list   ap;

	va_start(ap, fmt);
	vsyslog(LOG_DEBUG, fmt, ap);
	va_end(ap);
    }
#endif
}

#ifndef HAVE_STRERROR
extern const char *const sys_errlist[];

#define strerror(errnum) sys_errlist[errnum]
#endif

void 
logerrno(int priority, const char *fmt,...)
{
#if HAVE_VSNPRINTF
    char     *buffer;
    int       size = 1024,
              len;

    va_list   ap;

    va_start(ap, fmt);
    while (1) {
	if (!(buffer = malloc(size)))
	    return;
	len = vsnprintf(buffer, size - 1, fmt, ap);
	if (len <= size - 6) {	/* We need some space for the ': %s' */
	    strcat(buffer, ": %s");
	    log(priority, buffer, strerror(errno));
	    free(buffer);
	    break;
	}
	free(buffer);
	size += 1024;		/* Try again with larger buffer */
    }
    va_end(ap);
#else
    char      buffer[4096];

    va_list   ap;

    va_start(ap, fmt);
    vsprintf(buffer, fmt, ap);
    strcat(buffer, ": %s");
    log(priority, buffer, strerror(errno));
    va_end(ap);
#endif
}

/****i***************************************************************************
NAME
     xmalloc, xstrdup - conveniently allocate buffers

SYNOPSIS
     #include <support.h>

     void *
     malloc(const size_t size)

     char *
     malloc(const char *string)

DESCRIPTION
     xmalloc() does absolutely the same as malloc(3), except for the
     fact that it will terminate the program with exit(3) if the
     allocation was not successful. This releases you from the bother
     of always checking the return code.

     xstrdup() is an equivalent to strdup(3) and cannot fail either,
     as it does terminate the program with exit() in case of an error,
     also.

     While xmalloc() and xstrdup() may not always be appropriate, they
     are secure to use for most memory allocations. If a system can
     not provide 20 byte of free memory, there's something going
     seriously wrong anyway. :-)

RETURN
     Both routines can not fail. Well... actually they can, but you
     shouldn't notice.

BUGS
     none known

SEE ALSO
     malloc(3), strdup(3), atexit(3)
********************************************************************************/

void     *
xmalloc(size_t size)
{
    void     *buffer = malloc(size);

    if (!buffer) {
	log(LOG_ERR, "Failed allocating %d byte.", size);
	exit(10);
    }
    debug(3, "Allocated %d byte successfully.", size);
    return buffer;
}

/*
 * xstrdup() is a special case of strdup(), as it uses xmalloc() to
 * allocate the memory required for the string.
 */
char     *
xstrdup(const char *string)
{
    char     *buffer = xmalloc(strlen(string) + 1);

    strcpy(buffer, string);
    return buffer;
}
